package liviu.tudor.convertor;

import java.util.Stack;

import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.midlet.MIDlet;

/**
 * Our convertor midlet
 * 
 * @author Liviu Tudor
 */
public class ConvertorMidlet extends MIDlet
{
    /**
     * Current midlet instance.
     */
    private static ConvertorMidlet instance;
    
    /**
     * Current screen showing.
     */
    private Displayable displayable = null;
    
    /**
     * Stack of screens currently showed by this application
     */
    private Stack screens = new Stack();
  
    /** 
     * Constructor 
     */
    public ConvertorMidlet()
    {
        instance = this;
    }

    /** 
     * Main method 
     */
    public void startApp()
    {
        displayable = new ConvertorDisplayable();
        Display.getDisplay(this).setCurrent( displayable );
    }

    /** 
     * Handle pausing the MIDlet 
     */
    public void pauseApp()
    {
        notifyPaused();
    }

    /** 
     * Handle destroying the MIDlet 
     */
    public void destroyApp( boolean unconditional )
    {
        while( screens.size() > 0 )
        {
            Displayable d = (Displayable)screens.pop();
            closeScreen( d );
        }
        screens = null;
        closeScreen( displayable );
        displayable = null;
        notifyDestroyed();
    }
    
    protected static void closeScreen( Displayable d )
    {
        if( d instanceof ConvertorDisplayable )
        {
            ConvertorDisplayable cd = (ConvertorDisplayable)d;
            cd.cleanup();
        }
        else if( d instanceof InfoDialog )
        {
            InfoDialog id = (InfoDialog)d;
            id.cleanup();
        }
    }
    
    public static void openScreen( Displayable d )
    {
        instance.internalOpenScreen( d );
    }
    
    protected void internalOpenScreen( Displayable d )
    {
        screens.push( displayable );
        displayable = d;
        Display.getDisplay(this).setCurrent( displayable );
    }
    
    public static void closeCurrentScreen()
    {
        instance.internalCloseCurrentScreen();
    }
    
    protected void internalCloseCurrentScreen()
    {
        if( screens.size() == 0 )
        {
            quitApp();
            return;
        }
        closeScreen( displayable );
        displayable = (Displayable)screens.pop();
        Display.getDisplay(this).setCurrent( displayable );
    }
    
    public static void showMessage( String title, String message )
    {
        InfoDialog dlg = new InfoDialog( title, message );
        openScreen( dlg );
    }

    /** 
     * Quit the MIDlet 
     */
    public static void quitApp()
    {
        instance.destroyApp(true);
        instance.notifyDestroyed();
        instance = null;
        System.gc();    // free up resources if this is the right time to do so
    }
}
