package liviu.tudor.convertor;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.ItemStateListener;
import javax.microedition.lcdui.TextField;

import liviu.tudor.convertor.provider.ConvertProvider;

public class ProviderInput extends Form implements CommandListener
{
    public static int           MAX_CHARS = 20;
    public static int           MAX_DECIMALS = 10;
    
    protected ConvertProvider   conv;
    protected TextField[]       inputs;
    protected int               currCtrl;
    protected double            mask;
    protected ItemStateListener stateLsnr;
    
    public ProviderInput() 
    {
        super( "Convertor" );
        conv = null;
        inputs = null;
        currCtrl = -1;
        setDecimals( MAX_DECIMALS );
        setCommandListener( this );
        stateLsnr = new CheckState();
        setItemStateListener( stateLsnr );
        addCommand( new Command("Close", Command.EXIT, 1) );
    }

    protected void setDecimals( int decimals )
    {
        mask = 1.0;
        for( int i = 0; i < decimals; i++ )
            mask *= 10.0;
    }
    
    protected String truncate( String s )
    {
        if( s == null )
            return null;
        if( s.length() > MAX_CHARS )
            s = s.substring( 0, MAX_CHARS - 4 ) + "...";
        return s;
    }
    
    protected String format( double d )
    {
        /*
         * Eliminate zeros straight away
         */
        if( d == 0.0 )
            return "0";
        
        /*
         * Eliminate integer values
         */
        long l = (long)d;
        String res = String.valueOf(l);
        if( d -l == 0.0 )
            return truncate( res );
        
        /*
         * So we have some decimals.
         * Go through each digit one at a time
         */
        //add the point first
        res += '.';
        //now find out how many chars have we really got left
        int allowed = Math.min( MAX_DECIMALS, MAX_CHARS - res.length() );
        //get the decimal part
        d -= l;
        int digit = 0,
            countDigits = 0;
        while( (d != 0.0) && (countDigits < allowed) )   //repeat until we run out of digits
        {
            //get the next digit
            d *= 10.0;
            digit = (int)d;
            res += String.valueOf(digit);
            d -= digit;
            countDigits++;
        }
        return res;
    }
    
    public void setProvider( ConvertProvider conv )
    {
        this.conv = conv;
        updateFields();
    }
    
    /**
     * Go through the fields of the current convertor and update the form GUI
     */
    protected void updateFields()
    {
        /*
         * Delete what was there previously and set title
         */
        deleteAll();
        inputs = null;
        currCtrl = -1;
        if( conv == null )
        {
            setTitle( "Convertor" );
            return;
        }
        setTitle( conv.getName() );
        
        /*
         * Now iterate through the fields
         */
        String fields[] = conv.getFieldNames();
        inputs = new TextField[fields.length];
        for( int i = 0; i < fields.length; i++ )
        {
            inputs[i] = new TextField( fields[i], "", MAX_CHARS, TextField.DECIMAL );
            append( inputs[i] );
        }
    }
    
    protected void updateFieldValues()
    {
        double []vals = conv.getFieldValues();
        for( int i = 0; i < vals.length; i++ )
            if( i != currCtrl )
                inputs[i].setString( format(vals[i]) );
    }

    public void commandAction( Command c, Displayable d ) 
    {
        if( d != this )
            return;
        if( c.getCommandType() == Command.EXIT || 
            c.getCommandType() == Command.CANCEL ||
            c.getCommandType() == Command.OK ||
            c.getCommandType() == Command.STOP )
        {
            //exiting
            setProvider( null );
            ConvertorMidlet.closeCurrentScreen();
            return;
        }
    }
    
    class CheckState implements ItemStateListener
    {
        public void itemStateChanged( Item item )
        {
            if( inputs == null || conv == null )
                return;
            if( !(item instanceof TextField) )
                return;
            for( int i = 0; i < inputs.length; i++ )
            {
                if( inputs[i] == item )
                {
                    /*
                     * so we found our item -- now can we parse the input as a double?
                     */
                    double d = 0;
                    try
                    {
                        if( !"".equals(inputs[i].getString()) )
                            d = Double.parseDouble( inputs[i].getString() );
                    }
                    catch( NumberFormatException nfe )
                    {
                        ConvertorMidlet.showMessage( "Invalid Data", "You must enter a valid number in this field!" );
                        return;
                    }
                    currCtrl = i;
                    try
                    {
                        conv.setFieldValue( i, d );
                    }
                    catch( IllegalArgumentException iae )
                    {
                        ConvertorMidlet.showMessage( "Wrong Data", "Value " + d + " is not accepted in this field!" );
                        return;
                    }
                    updateFieldValues();
                    return;
                }
            }
        }
    }
}
