package liviu.tudor.convertor.provider;

/**
 * This class converts areas
 * 
 * @author Liviu Tudor
 */
public class SpeedConvertor extends ConvertProviderAdapter 
{
    /**
     * Constructor.
     * Here we initialize all of our values we can convert.
     * The reference value is m/s.
     */
    public SpeedConvertor() 
    {
        addField( "Kilometers/hour",    0.27778 );
        addField( "Meters/second",      1, true );
        addField( "Miles/hour",         0.44694 );
        addField( "Knots",              0.51444 );
        addField( "Foot/second",        0.3048 );
        addField( "Inch/second",        0.0254 );
        addField( "Mach",               340.29 );
        addField( "Speed of light (vacuum)", 299792458.0 );
        addField( "Speed of sound",     340.29 );
        addField( "Benz",               1.0 );
    }

    /**
     * Return our convertor image name -- in this case, a speedometer
     */
    public String getImageName() 
    {
        return "/img/speed.png";
    }

    /**
     * Check the value.
     * In this case we simply want a positive value
     */
    protected void checkValue(int index, double value) throws IllegalArgumentException 
    {
        if( value < 0 )
            throw new IllegalArgumentException( "Only positive values are converted!" );
    }

    /**
     * Name this convertor <i>Speed</i>
     */
    public String getName() 
    {
        return "Speed";
    }

    public String getInfo() 
    {
        return "Provides speed conversions to and from: m/s, kmph, mph etc.";
    }

}
