package liviu.tudor.convertor.provider;

/**
 * This class converts weights
 * 
 * @author Liviu Tudor
 */
public class WeightConvertor extends ConvertProviderAdapter 
{
    /**
     * Constructor.
     * Here we initialize all of our values we can convert.
     * The reference value is kilogram.
     */
    public WeightConvertor() 
    {
        addField( "Microgram",  1.0E-9 );
        addField( "Milligram",  0.000001 );
        addField( "Centigram",  0.00001 );
        addField( "Decigram",   0.0001 );
        addField( "Gram",       0.001 );
        addField( "Dekagram",   0.01 );
        addField( "Hectogram",  0.1 );
        addField( "Kilogram",   1, true );
        addField( "Tonne",      1000 );
        addField( "Pound (lb)", 0.45359237 );
        addField( "Ounce",      0.028349523 );
        addField( "Stone",      6.35029 );
        addField( "Dram",       0.001771845 );
    }

    /**
     * Check the value.
     * In this case we simply want a positive value
     */
    protected void checkValue(int index, double value) throws IllegalArgumentException 
    {
        if( value < 0 )
            throw new IllegalArgumentException( "Only positive values are converted!" );
    }

    /**
     * Return our convertor image name -- in this case, a scale
     */
    public String getImageName() 
    {
        return "/img/weight.png";
    }

    /**
     * Name this convertor <i>Weight</i>
     */
    public String getName() 
    {
        return "Weight";
    }

    public String getInfo() 
    {
        return "Provides weight conversions to and from: kg, stone, ounce, pound etc.";
    }

}
